package nl.uvt.locator;

import java.io.File;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;

import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Nodes;
import nu.xom.xslt.XSLTransform;


public class MerescoConnection {

	List<LinkInfo> query( QueryData pQ ) throws LocatorException {
		List<LinkInfo> meResult = null;
		nu.xom.Document result;
		String url = Locator.locatorConfig.getProperty("merescoUri");
		String query = "";
		try {
			Properties prop = new Properties( Locator.sruDefaultProps );
			prop.setProperty("recordSchema","mods" );
			if ( pQ.epn.length() > 0 ){
				Locator.logger.debug("direct search with EPN: " + pQ.epn );
				query = "mods.location.holdingSimple.copyInformation.note=" + pQ.epn;						
			}
			else if ( pQ.genre.equals("book") || pQ.genre.equals("bookitem")){
				String isbnPart = "";
				String locationPart = "";
				if ( pQ.isbn.length() > 0 ){
					Locator.logger.debug("ISBN: " + pQ.isbn );
					isbnPart = "mods.identifier=" + pQ.isbn;						
				}
				if ( pQ.location.length() > 0 ){
					String location = fixLocation( pQ.location );
					locationPart = "mods.location.holdingSimple.copyInformation.shelfLocator=\"" + location + "\"";
				}
				if ( !isbnPart.equals("") || !locationPart.equals("") ){
					if ( isbnPart.equals("") )
						query = locationPart;
					else if ( locationPart.equals("") )
						query = isbnPart;
					else
						query = isbnPart + " or " + locationPart;
				}
				else if ( pQ.atitle.length() > 0 ) {
					query = "mods.titleInfo.title=\"" + pQ.atitle + "\"";
				}
				else if ( pQ.title.length() > 0 ) {
					query = "mods.titleInfo.title=\"" + pQ.title + "\"";
				}
				else {
					// fails when no title or issn available
					return meResult;
				}
			}
			else {
				String issnPart = "";
				String locationPart = "";
				if ( pQ.issn.length() > 0 ){
					issnPart = "mods.identifier=" + pQ.issn;						
				}
				if ( pQ.location.length() > 0 ){
					String location = fixLocation( pQ.location );
					locationPart = "mods.location.holdingSimple.copyInformation.shelfLocator=\"" + location + "\"";
				}
				if ( !issnPart.equals("") || !locationPart.equals("") ){
					if ( issnPart.equals("") )
						query = locationPart;
					else if ( locationPart.equals("") )
						query = issnPart;
					else
						query = issnPart + " or " + locationPart;
				}
				else if ( pQ.title.length() > 0 ) {
					query = "mods.titleInfo.title=\"" + pQ.title + "\"";
				}
				else if ( pQ.atitle.length() > 0 ) {
					query = "mods.titleInfo.title=\"" + pQ.atitle + "\"";
				}
				else
					// fails when no title or issn available
					return meResult;
			}
			query += " and meta.repository.repositoryGroupId exact opc-uvt-nl";
			prop.setProperty("query", query );
			url = HttpConnection.createUrlString(url, prop );
			Locator.logger.debug("Try URL: " + url );
			result = HttpConnection.httpGetDocument( url, Locator.httpConfig );
			int hits = Locator.extractHits( result );
			Locator.logger.debug("meresco numOfRecords=" + hits );
			if ( hits > 0 ){
//				Locator.logger.debug( "meresco doc:" + result.toXML());
				String sheetName = Locator.locatorConfig.getProperty("merescoSheet");
				meResult = getModsResults( result, pQ, sheetName );
			}
		}
		catch ( Exception e) {
			throw new LocatorException( "meresco URL: '" + url + "' failed"); 
		}
		return meResult;
	}

	private String fixLocation( String loc ) {
		String result = loc.replaceAll("/\\d{4}.*", "/...");
		if ( !loc.equals(result))
			Locator.logger.debug("fixLocation replaced '" + loc + "' by: '" + result + "'" );			
		return result;
	}


	static public List<LinkInfo> getModsResults( Document doc, QueryData pQ, String sheetName ){
		List<LinkInfo> meResult = null;
		Builder builder = new Builder();
		try {
			HoldingInfo.modsContext.addNamespace("mods", HoldingInfo.modsNamespace);
			HoldingInfo.modsContext.addNamespace("xsi", HoldingInfo.schemaNamespace);
			HoldingInfo.modsContext.addNamespace("a", HoldingInfo.extNamespace);
			Locator.logger.debug( "use xslt sheet:" + sheetName );
			File f = Locator.getFile( sheetName );
			if ( f != null ){
				nu.xom.Document stylesheet = builder.build(f);
				XSLTransform transform = new XSLTransform(stylesheet);
//				Locator.logger.debug( "input doc:" + doc.toXML());
				Nodes output = transform.transform( doc );
				nu.xom.Document standardDoc = XSLTransform.toDocument(output);
//				Locator.logger.debug( "transformed doc:" + standardDoc.toXML());
				Nodes records = standardDoc.query("//mods:mods", HoldingInfo.modsContext);
				if ( records.size() > 0 ) {
					meResult = new LinkedList<LinkInfo>();
					Locator.logger.debug("found " + records.size() + " real records");
					for ( int i=0; i < records.size(); ++i ){
						// every record may contain several occurrences
						// split these in separate records
						Locator.logger.debug(" try record: " + (i+1)  );
						HoldingInfo.fillMultipleRecord( records.get(i), meResult );
					}
				}
			}
			else {
				Locator.logger.fatal("unable to get a stylesheet:" + sheetName );
			}
		}
		catch( Exception e )
		{
			Locator.logger.error( "meresco exception", e );
		}
		return meResult;
	}

}
